﻿using System;
using System.Collections.Generic;
using System.Web.UI.WebControls;
using Infragistics.Web.UI.NavigationControls;


namespace EnsureTemplate.ClassFiles
{
    /// <summary>
    /// CurriculumElementDataTreeManager Class
    /// </summary>
    public static class CurriculumElementDataTreeManager
    {



        #region internal instance methods

        /// <summary>
        /// EnableDataTree
        /// Enables/disables all the controls in the datatree
        /// </summary>
        /// <param name="wdt"></param>
        /// <param name="cVersion"></param>
        /// <param name="enable"></param>
        internal static void EnableDataTree(WebDataTree wdt, bool enable)
        {
            foreach (DataTreeNode thisNode in wdt.Nodes)
            {
                EnableNodeControls(thisNode, enable);
            }
            wdt.Enabled = enable;
            wdt.DragDropSettings.EnableDragDrop = enable;
        }

        /// <summary>
        /// Removes all nodes from the WebDataTree
        /// </summary>
        internal static void ClearDataTree(WebDataTree wdt)
        {
            List<DataTreeNode> nodesToDelete = new List<DataTreeNode>();
            foreach (DataTreeNode thisNode in wdt.Nodes)
            {
                nodesToDelete.Add(thisNode);
            }
            foreach (DataTreeNode nodeToDelete in nodesToDelete)
            {
                wdt.Nodes.Remove(nodeToDelete);
            }
        }

        internal static void PopulateCurriculumRequirementsDataTree(WebDataTree wdt)
        {
            ClearDataTree(wdt);
            Guid newGuid = Guid.NewGuid();
            RequirementNode reqNode = new RequirementNode(newGuid);
            reqNode.AddToTree(wdt);
            reqNode.Expanded = true;
            reqNode.ExpandChildren();

        }

        internal static DataTreeNode FindNodeByElementId(WebDataTree wdt, Guid elementId)
        {
            foreach (DataTreeNode thisNode in wdt.Nodes)
            {
                if (thisNode.Key.Equals(elementId.ToString("n").ToUpper()))
                {
                    return thisNode;
                }
                DataTreeNode maybeTheNode = FindChildNodeNodeByElementId(thisNode, elementId);
                if (maybeTheNode != null)
                {
                    return maybeTheNode;
                }
            }
            return null;
        }

        internal static void SelectNodeByElementId(WebDataTree wdt, Guid elementId)
        {
            DeselectAllNodes(wdt);
            DataTreeNode nodeToSelect = FindNodeByElementId(wdt, elementId);
            if (nodeToSelect != null)
            {
                nodeToSelect.Selected = true;
            }
        }

        internal static void DeselectAllNodes(WebDataTree wdt)
        {
            DeselectAllNodesInCollection(wdt.Nodes, true);
        }

        /// <summary>
        /// Returns requirement ids and whether or not the node for each requirement is expanded in the tree
        /// </summary>
        /// <param name="wdt"></param>
        /// <returns></returns>
        internal static Dictionary<Guid, bool> GetExpansionStateDictionary(WebDataTree wdt)
        {
            Dictionary<Guid, bool> expandedStateDictionary = new Dictionary<Guid, bool>();
            foreach (DataTreeNode dtn in wdt.Nodes)
            {
                AddItemsToExpandedStateDictionary(dtn, ref expandedStateDictionary);
            }

            return expandedStateDictionary;
        }

        internal static void SetExpansionsForTree(WebDataTree wdt, Dictionary<Guid, bool> nodeExpansionStates)
        {
            foreach (Guid thisKey in nodeExpansionStates.Keys)
            {
                DataTreeNode thisNode = FindNodeByElementId(wdt, thisKey);
                if (thisNode != null)
                {
                    thisNode.Expanded = nodeExpansionStates[thisKey];
                }
            }
        }

        #endregion


        #region private methods

        private static void DeselectAllNodesInCollection(DataTreeNodeCollection nodes, bool childrenToo)
        {
            foreach (DataTreeNode thisNode in nodes)
            {
                thisNode.Selected = false;
                if (childrenToo)
                {
                    foreach (DataTreeNode childNode in thisNode.Nodes)
                    {
                        DeselectAllNodesInCollection(childNode.Nodes, true);
                    }
                }
            }
        }

        private static void EnableNodeControls(DataTreeNode thisNode, bool enable)
        {
            bool versionIsPublished = false;
            string[] imageButtonIds = { "ibEditRequirement", "ibRemoveRequirement", "ibEditElement", "ibRemoveElement", "ibAdd" };
            foreach (string thisImageButtonId in imageButtonIds)
            {
                ImageButton ib = thisNode.FindControl(thisImageButtonId) as ImageButton;
                if (ib != null)
                {
                    bool enableThisOne = false;
                    if (thisImageButtonId.Equals("ibAdd"))
                    {
                        enableThisOne = enable && (!versionIsPublished || MayAddToThisNode(thisNode));
                    }
                    else
                    {
                        enableThisOne = enable && (!thisImageButtonId.Contains("Remove") || NodeMayBeRemoved(thisNode));
                    }
                }
            }

            foreach (DataTreeNode childNode in thisNode.Nodes)
            {
                EnableNodeControls(childNode, enable);
            }
        }

        private static bool MayAddToThisNode(DataTreeNode thisNode)
        {
            bool returnValue = true;
            return returnValue;
        }

        private static bool NodeMayBeRemoved(DataTreeNode thisNode)
        {
            bool returnValue = true;

            return returnValue;
        }

        private static DataTreeNode FindChildNodeNodeByElementId(DataTreeNode node, Guid elementId)
        {
            foreach (DataTreeNode thisNode in node.Nodes)
            {
                if (thisNode.Key.Equals(elementId.ToString("n").ToUpper()))
                {
                    return thisNode;
                }
                DataTreeNode maybeTheNode = FindChildNodeNodeByElementId(thisNode, elementId);
                if (maybeTheNode != null)
                {
                    return maybeTheNode;
                }
            }
            return null;
        }


        /// <summary>
        /// Add expansion state of requirement nodes and subrequirement nodes to the dictionary
        /// </summary>
        /// <param name="dtn"></param>
        /// <param name="expandedStateDictionary"></param>
        private static void AddItemsToExpandedStateDictionary(DataTreeNode dtn, ref Dictionary<Guid, bool> expandedStateDictionary)
        {
            if (dtn.Value.StartsWith("Requirement"))
            {
                expandedStateDictionary.Add(new Guid(dtn.Key), dtn.Expanded);
                foreach (DataTreeNode subNode in dtn.Nodes)
                {
                    AddItemsToExpandedStateDictionary(subNode, ref expandedStateDictionary);
                }
            }
        }

        #endregion

    }
}