﻿using AjaxControlToolkit;
using System;
using System.Linq;
using System.Collections.Generic;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using Infragistics.Web.UI.NavigationControls;

namespace EnsureTemplate.ClassFiles
{
    internal class RequirementNode : DataTreeNode
    {
        private const string _templateId = "RequirementTemplate";
        private readonly int _requirementItemCount = 0;

        internal RequirementNode(Guid fakeThing)
        {

            _requirementItemCount = 4;
            Key = fakeThing.ToString("n").ToUpper();
            Text = fakeThing.ToString("n").ToUpper();
            // the Value property will be used buy the javascript code to get properties about the node that are not built in to a generic DataTreeNode
            Value = "Requirement";
            CssClass = RequirementIsComplete ? "requirementNode" : "requirementNodeIncomplete";
        }


        internal bool RequirementIsEmpty
        {
            get
            {
                return RequirementItemCount.Equals(0);
            }
        }

        internal int RequirementItemCount
        {
            get
            {
                return _requirementItemCount;
            }
        }

        internal bool RequirementIsComplete
        {
            get
            {
                return (true);
            }
        }

        #region internal methods

        /// <summary>
        /// Adds a requirement and all of its SubRequirements and Elements to a datatree
        /// </summary>
        /// <param name="tree"></param>
        internal void AddToTree(WebDataTree tree)
        {
            tree.Nodes.Add(this);
            SetupTemplate();
            AddChildNodes();
        }

        #endregion

        #region private methods

        private void AddChildNodes()
        {
            ElementNode childElementNode1 = new ElementNode(Guid.NewGuid());
            childElementNode1.AddToNode(this);

            ElementNode childElementNode2 = new ElementNode(Guid.NewGuid());
            childElementNode2.AddToNode(this);
            
            ElementNode childElementNode3 = new ElementNode(Guid.NewGuid());
            childElementNode3.AddToNode(this);

            ElementNode childElementNode4 = new ElementNode(Guid.NewGuid());
            childElementNode4.AddToNode(this);
            // put elements and subrequirements into the same list so we can sort them regardless of typeSubRequirements
            //List<CurriculumElement> allChildElements = Requirement.Elements.OfType<CurriculumElement>().ToList<CurriculumElement>();
            //allChildElements.AddRange(Requirement.SubRequirements.OfType<CurriculumElement>().ToList<CurriculumElement>());
            //allChildElements.Sort();

            //foreach (CurriculumElement elementToAdd in allChildElements)
            //{
            //    if (elementToAdd.BasicType.Equals(ElementType.Requirement))
            //    {
            //        RequirementNode childReqNode = new RequirementNode((CurriculumRequirement)elementToAdd);
            //        Nodes.Add(childReqNode);
            //        childReqNode.SetupTemplate();
            //        childReqNode.AddChildNodes();
            //    }
            //    else
            //    {
            //        ElementNode childElementNode = new ElementNode(elementToAdd);
            //        childElementNode.AddToNode(this);
            //    }
            //}
        }

        private string RequirementTypeLabelText()
        {
            string returnValue = "Complete all ";

            //double totalUnits = RequirementValidator.GetUnitsContained(Requirement);
            //string returnValue = string.Empty;
            //switch (Requirement.RequirementType.Name)
            //{
            //    case "All":
            //        returnValue = "Complete all " + totalUnits.ToString();
            //        break;
            //    case "Ordered":
            //        returnValue = String.Format("Complete all {0} in order", totalUnits);
            //        break;
            //    case "N of X":
            //        returnValue = String.Format("Complete {0} of {1}", Requirement.RequirementCondition, totalUnits);
            //        break;
            //    case "Number of Credits":
            //        returnValue = String.Format("Complete {0} {1} of {2}", Requirement.RequirementCondition, Requirement.RequirementCondition.Equals(1) ? "credit" : "credits", totalUnits);
            //        break;
            //    case "Number of Hours":
            //        returnValue = String.Format("Complete {0} {1} of {2}", Requirement.RequirementCondition, Requirement.RequirementCondition.Equals(1) ? "hour" : "hours", totalUnits);
            //        break;
            //    case "Recommended":
            //        returnValue = String.Format("Recommended ({0})", totalUnits);
            //        break;
            //    default:
            //        returnValue = "Undefined Reqiurement Type";
            //        break;
            //}

            return returnValue;
        }

        /// <summary>
        /// This can only be done AFTER the node has been added to the tree
        /// </summary>
        private void SetupTemplate()
        {
            TemplateId = _templateId;
            EnsureTemplate();

            // set the labels
            Label lblTitle = FindControl("lblName") as Label;
            lblTitle.Text = "Requirement.Name";
            lblTitle.ToolTip = "Requirement.Description";

            //if (Requirement.Parent is CurriculumRequirement)
            //{
            //    CurriculumRequirement parentReq = Requirement.Parent as CurriculumRequirement;
            //    if (parentReq.RequirementType.Name.Equals("Ordered"))
            //    {
            //        lblTitle.Text = String.Format("{0}. {1}", Requirement.SortValue + 1, lblTitle.Text);
            //    }
            //}

            if (!RequirementIsComplete)
            {
                lblTitle.ForeColor = System.Drawing.Color.Red;
            }

            Label lblType = FindControl("lblType") as Label;
            lblType.Text = RequirementTypeLabelText();

            // setup the image buttons
            string commandArgument = Key;
            bool showAddRequirement = true;

            ImageButton ibEdit = FindControl("ibEditRequirement") as ImageButton;
            ibEdit.CommandArgument = commandArgument;

            ImageButton ibRemove = FindControl("ibRemoveRequirement") as ImageButton;
            ibRemove.CommandArgument = commandArgument;
            ConfirmButtonExtender cbe = FindControl("cbeRemoveRequirement") as ConfirmButtonExtender;
            cbe.ConfirmText = String.Format("Are you sure you want to delete \"{0}\"?", Key);

            // set up the popmenu and links therein for adding elements and requirements.
            HtmlGenericControl div = FindControl("divAddRequirement") as HtmlGenericControl;
            div.Visible = showAddRequirement;

            LinkButton lbAddReq = FindControl("lbAddRequirement") as LinkButton;
            lbAddReq.CommandArgument = commandArgument;

            LinkButton lbAddEl = FindControl("lbAddGlobalElement") as LinkButton;
            lbAddEl.CommandArgument = commandArgument;

            LinkButton lbAddLocalEl = FindControl("lbAddLocalElement") as LinkButton;
            lbAddLocalEl.CommandArgument = commandArgument;

            Panel pnlHoverMenu = FindControl("pnlHoverMenuAdd") as Panel;
            if (pnlHoverMenu != null)
            {
                pnlHoverMenu.Style.Add("display", "none");
            }            
        }
        #endregion

    }
}